import os
import json
from .logger import setup_logger

cache_file_dir = os.path.join(os.getcwd(), ".cache")
cache_file = os.path.join(cache_file_dir, "dipperai.json")
logger = setup_logger(
    debug=True if os.environ.get("dipperai_debug_model", None) else False
)

class Cache:
    def __init__(self):
        self.cache_file = self.init_cache()
        self.cache_data = self.load_cache()
        self.change = False

    def save_cache(self):
        """
        persistence cache file.
        """
        if self.change:
            with open(self.cache_file, "w") as f:
                f.write(json.dumps(self.cache_data))
            logger.info("update cache file")

    def init_cache(self):
        """Init cache file, if exist return, else create cache file; cache file path: {run-path}/.cache/dipperai.json.

        :return:
        """
        if not os.path.exists(cache_file):
            os.makedirs(cache_file_dir, exist_ok=True)
            with open(cache_file, "w") as f:
                f.write("{}")
        return cache_file

    def load_cache(self) -> dict:
        """
        Load the cache file.
        """
        try:
            with open(cache_file) as f:
                cache_data = json.loads(f.read())
            return cache_data
        except Exception as e:
            logger.error(e)
            # If the file format is abnormal, the cache will be cleared
            os.remove(cache_file)
            self.init_cache()
            return {}

    def get_cache(self, key:str=None) -> dict:
        """Get the value by the key from the cache file.

        :param key:
        :return:
        """
        return self.cache_data.get(key, {})


    def set_cache(self, key:str, value:str) -> bool:
        """Set the key and the value to cache file.

        :param key:
        :param value:
        :return:
        """
        if self.cache_data.get(key, {}) != value:
            self.cache_data[key] = value
            self.change = True
        return True

class OperateCache:

    def __init__(self, cache:Cache):
        self.cache = cache
    
    def __enter__(self):
        return self.cache

    def __exit__(self, exc_type, exc_val, exc_tb):
        self._save_to_file()
    
    def _save_to_file(self):
        cache.save_cache()

# 全局cache对象, 相当于单例模式，只有在第一次调用时初始化
# 避免每次调用都初始化，提高性能
cache = Cache()


import os
import sys
import importlib
import requests
from .core import MaaS
from ..vendor import Alibaba


class HuggingFace(MaaS):
    def __init__(self, model_id: str, model_version: str = None, config: dict = None, 
                  service_config: dict = None, cloud: any = None, service_url:str = None):
        self.access_token = os.environ.get("HF_ACCESS_TOKEN", "")
        self.hf_endpoint = os.environ.get("HF_ENDPOINT") or "https://huggingface.co"
        super().__init__(model_id=model_id, model_version=model_version or "main", cloud=cloud, 
                         service_config=service_config, service_url=service_url)


    def get_model_meta(self):
        """
        从huggingface.co获取模型的元数据。
        
        该方法构造一个请求URL，然后发送GET请求以获取指定模型的元数据。
        
        参数:
        - self: 对象自身的引用，包含模型的ID和访问huggingface.co的端点。
        
        返回值:
        - 返回一个JSON对象，包含从huggingface.co获取到的模型元数据。
        """
        # 构造请求元数据的URL
        meta_url = f"{self.hf_endpoint}/api/models/{self.model_id}"
        # 发送GET请求获取元数据
        r = requests.get(meta_url)
        # 如果请求状态码不是200，抛出异常
        if r.status_code != 200:
            raise Exception("Failed to get model meta data from huggingface.co")
        # 返回解析后的JSON响应
        return r.json()
    
    def get_service_config(self, user_config: dict) -> dict:
        """
        get service config
        :return: dict
        """
        # 补充配置
        config_func_name = self.get_config_func_name()
        lib_module = importlib.import_module('DipperAI_inner.resources.config')
        return getattr(lib_module, config_func_name)(user_config, name=self.get_resource_name(),
            model_id=self.model_id, model_task=self.__get_task(), access_token=self.access_token, library=self.__get_library())

    def __get_library(self) -> str:
        """
        获取模型库地址
        """
        return self.model_meta["library_name"]

    def __get_task(self) -> str:
        """
        获取任务标识

        本方法用于从模型元数据中提取出pipeline标签，作为当前任务的标识。

        参数:
        self - 对象本身的引用。

        返回值:
        str - model_meta字典中"pipeline_tag"键对应的值，表示当前任务的标识。
        """
        return self.model_meta["pipeline_tag"]

        



import logging
from colorama import init, Fore, Style

init(autoreset=True)

# define a new log level
SUCCESS = 25
logging.addLevelName(SUCCESS, "SUCCESS")
DEFAULT_FORMAT = "[%(asctime)s] [%(levelname)s]] "


def success(self, message, *args, **kws):
    if self.isEnabledFor(SUCCESS):
        self._log(SUCCESS, message, args, **kws)


logging.Logger.success = success


class CustomFormatter(logging.Formatter):
    """define custom formatter.

    # grey: debug
    # green: success
    # yellow: warning
    # red: error.
    """

    grey = Style.DIM + Fore.WHITE
    green = Fore.GREEN
    yellow = Fore.YELLOW
    red = Fore.RED
    reset = Style.RESET_ALL

    def format(self, record):
        """Rewrite format style.

        :param record: record attr
        :return: message with new style.
        """
        log_fmt = DEFAULT_FORMAT
        if record.levelno == SUCCESS:
            log_fmt += f"{self.green}%(message)s{self.reset}"
        elif record.levelno == logging.INFO:
            log_fmt += "%(message)s"
        elif record.levelno == logging.WARNING:
            log_fmt += f"{self.yellow}%(message)s{self.reset}"
        elif record.levelno == logging.ERROR:
            log_fmt += f"{self.red}%(message)s{self.reset}"
        elif record.levelno == logging.CRITICAL:
            log_fmt += f"{self.red}%(message)s{self.reset}"
        else:
            log_fmt += "%(message)s"
        formatter = logging.Formatter(log_fmt, "%Y-%m-%d %H:%M:%S")
        record.message = formatter.format(record)
        return record.message


def setup_logger(debug=False):
    """Set logger attr.

    :param debug: if True, will show debug message, else will not show debug message
    :return: logger attr.
    """
    logger = logging.getLogger(__name__)
    logger.setLevel(logging.DEBUG if debug else logging.INFO)
    # 检查是否已经添加了处理程序
    if not logger.handlers:
        console_handler = logging.StreamHandler()
        console_handler.setFormatter(CustomFormatter())
        logger.addHandler(console_handler)
    else:
        # 可选：更新现有处理程序的日志级别
        for handler in logger.handlers:
            handler.setLevel(logging.DEBUG if debug else logging.INFO)

    return logger